
/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Cocoa/Cocoa.h>
#import "Controller.h"
#import "IDLViewer.h"
#import "IDLFunction.h"
#import "IDLParser.h"
#import "ObjectCell.h"
#import "IDLRunController.h"
#import "ResponseSimulator.h"
#import <Foundation/NSDebug.h>
#import "TiledBox.h"
#import "IDLLog.h"
#import "IDLNameService.h"
#import "ORB.h"

@implementation Controller

static NSDictionary* defaults = nil;
static IDLLog* _log = nil;
static BOOL useLog = YES;

static BOOL VXOpenHelpWithHelpViewer = NO;

+ (void) initialize
{
    if([Controller class] == self){
        //NSString* path = [[NSBundle mainBundle]pathForResource:@"defaults" ofType:@"plist"];
        defaults = [[NSDictionary alloc]initWithObjectsAndKeys:
			[NSNumber numberWithBool:YES], @"useLog", nil];
        NSDebugEnabled = [[NSUserDefaults standardUserDefaults]boolForKey:@"NSDebugEnabled"];
        VXOpenHelpWithHelpViewer = [[NSUserDefaults standardUserDefaults]boolForKey:@"VXOpenHelpWithHelpViewer"];
		useLog = [[NSUserDefaults standardUserDefaults]boolForKey:@"useLog"];
		if(useLog)
			_log = [[IDLLog allocWithZone:[self zone]]init];
    }
}

- (void) awakeFromNib
{
    NSBundle* bundle = [NSBundle mainBundle];
    NSDictionary* info = [bundle infoDictionary];
    NSString* version = [info objectForKey:@"CFBundleShortVersionString"];
    if(version)
        [versionField setStringValue:version];
}

- (void) newBrowser:sender
{
	IDLParser* parser = [[IDLParser alloc]init];
	IDLViewer* ctrl = [[IDLViewer alloc]initWithParser:parser];
	[parser release];
	[[ctrl window]makeKeyAndOrderFront:self];
}

static int tmpCount = 0;

- (void) saveTemplate:(NSDictionary*)dict
{
    NSString* temp = NSTemporaryDirectory();
    int ok = NO;
    if(temp){
        NSString* name = [dict objectForKey:@"name"];
        NSString* file = name?
            [NSString stringWithFormat:@"%@.plist",name] 
            :[NSString stringWithFormat:@"idle_pl%03d.plist",++tmpCount];
        NSString* path = [temp stringByAppendingPathComponent:NSUserName()];
        [[NSFileManager defaultManager]createDirectoryAtPath:path withIntermediateDirectories:YES attributes:nil error:NULL];
        path = [path stringByAppendingPathComponent:file];
        if([dict writeToFile:path atomically:YES]){
            ok = [[NSWorkspace sharedWorkspace]openFile:path];
           // [[NSFileManager defaultManager]removeFileAtPath:path handler:nil];
        }else
            NSRunAlertPanel(@"Alert",@"Cannot save property list",
                @"OK",nil,nil);
    }else
        NSRunAlertPanel(@"Alert", @"No temporary directory",
            @"OK",nil,nil);
}

- (void) applicationDidFinishLaunching:(NSNotification *)aNotification
{
    NSDictionary* bundles = [defaults objectForKey:@"bundles"];
    NSEnumerator* en = [[defaults objectForKey:@"loaded"] objectEnumerator];
    NSString* bundleName;
    NSString* componentName;
    NSBundle* mainBundle = [NSBundle mainBundle];
	
    //[NSBundle stripAfterLoading:NO];
    while(bundleName = [en nextObject]){
        NSString* path = [mainBundle pathForResource:bundleName ofType:@"bundle"];
        if(path){
            NSBundle* bundle = [NSBundle bundleWithPath:path];
            NSEnumerator* cen = [[bundles objectForKey:bundleName]objectEnumerator];
            NSString* className;
            while(className = [cen nextObject]){
                id class = [(NSBundle*)bundle classNamed:className];
                [[class alloc]init];
            }
        }
    }

    en = [[defaults objectForKey:@"components"] objectEnumerator];
    while(componentName = [en nextObject]){
        NSEnumerator* cen = [[bundles objectForKey:componentName]objectEnumerator];
        NSString* className;
        while(className = [cen nextObject]){
            id class = NSClassFromString(className);
            [[class alloc]init];
        }
    }
    [ORB ORBInit];
    [self newBrowser:self];
}

- (NSApplicationTerminateReply) applicationShouldTerminate:(NSApplication*)app
{
    NSArray* windows = [app windows];
    NSEnumerator* en = [windows objectEnumerator];
    NSWindow* win;
    BOOL canTerminate = YES;
    [[NSUserDefaults standardUserDefaults]synchronize];
    while(win = [en nextObject]){
        if([win isDocumentEdited]){
            canTerminate = NO;
            break;
        }
    }
    if(!canTerminate){
        BOOL ok = NSRunCriticalAlertPanel(@"Alert", @"There are unsaved idl models",
            @"Cancel", @"Quit anyway", nil);
        switch(ok){
            case NSAlertDefaultReturn:
                return NSTerminateCancel;
            case NSAlertAlternateReturn:
                return NSTerminateNow;
            default:
                return NSTerminateCancel;
        }
    }else
        return NSTerminateNow;
}

- (void) open:sender
{
	NSOpenPanel* openPanel = [NSOpenPanel openPanel];
	int ok = [openPanel runModalForTypes:[NSArray arrayWithObjects:@"idlcnf", @"idlmodel", nil]];
	if(ok){
		NSEnumerator* en = [[openPanel filenames]objectEnumerator];
		NSString* path;
		while(path = [en nextObject]){
			if([[path pathExtension]isEqualToString:@"idlcnf"]){
				NSDictionary* dict = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
				if(dict){
					IDLParser* parser = [[IDLParser alloc]initByParsingIDLFiles:[dict objectForKey:@"files"]
							searchPaths:[dict objectForKey:@"searchPaths"]];
					IDLViewer* ctrl = [[IDLViewer alloc]initWithParser:parser];
					[parser release];
					[[ctrl window]makeKeyAndOrderFront:self];
				}
			}else
			if([[path pathExtension]isEqualToString:@"idlmodel"]){
	
			}
		}
	}
}

- (void) clearLog:sender
{
    [_log clearLog:sender];
}

- (void) showLog:sender
{
    [_log orderFront:self];
}

- (void) openNamingService:sender
{
    [[IDLNameService nameService]orderFront:self];
}

- (void) showSimulator:sender
{
	ResponseSimulator* simulator = [ResponseSimulator createSimulator];
	[[simulator window]makeKeyAndOrderFront:self];
}

/*
extern OSStatus AHGotoPage(NSString* book, NSString* pageUrl, NSString* anchor);

- (void)showHelpPage:(NSString*)page
{
    NSBundle* bundle = [NSBundle mainBundle];
    NSString* path = [bundle pathForResource:@"IDLBHelp" ofType:@""];
    if(path){
        NSString* file = [path stringByAppendingPathComponent:page];
        NSString* urlString = [NSString stringWithFormat:@"file://%@", file];
        //OSStatus err = AHGotoPage(@"DABHelp", path, nil);
        OSStatus err = AHGotoPage(nil, urlString, nil);
        if(err != 0){
            NSLog(@"Unable to load help %@, error: %d\n", page, err);
            NSBeep();
        }
    }else
        NSBeep();
}
*/
- (void) showORBHelp:sender
{
    NSBundle* bundle = [NSBundle mainBundle];
    NSDictionary* info = [bundle infoDictionary];
    NSString* path = [bundle pathForResource:[info objectForKey:@"ORBHelpDirectory"] ofType:@""];
    NSString* appPath = [[NSWorkspace sharedWorkspace]fullPathForApplication:@"Help Viewer"];
    if(path){
        NSString* file = [path stringByAppendingPathComponent:[info objectForKey:@"ORBHelpFile"]];
        if(VXOpenHelpWithHelpViewer && appPath)
            [[NSWorkspace sharedWorkspace]openFile:file withApplication:appPath];
        else
            [[NSWorkspace sharedWorkspace]openFile:file];
    }
}

@end
