/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "IDLDefinition_Private.h"
#import "IDLEnum.h"
#import "ObjectCell.h"

@implementation IDLEnum(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"enum13"];
}
- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
    int n = 0;
    NSEnumerator* enumerator = [_strings objectEnumerator];
    NSString* item;

    while(item = [enumerator nextObject]){
        id cell;
        [matrix addRow];
        cell = [matrix cellAtRow:n++ column:0];
        [cell setStringValue:item];
        [cell setLoaded:YES];
        [cell setLeaf:YES];
		[cell setImage:[NSImage imageNamed:@"const13"]];
        [cell setRepresentedObject:[NSString stringWithFormat:@"%@ = %@",
                item,[(NSNumber*)[_enumNames objectForKey:item]stringValue]]];
    }
}

- (NSString*) title
{
    return @"enum";
}

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"enum %@",_name];
}

- (NSString*) view:(NSView*)view stringForToolTip:(NSToolTipTag)tag point:(NSPoint)p userData:(void*)data
{
    return [self stringValue];
}

@end

@implementation IDLEnum(Diagram)

- (void) fillItemMatrix:matrix typeMatrix:typeMatrix box:dbox;
{
    int n = 0;
    NSEnumerator* enumerator = [_strings objectEnumerator];
    NSString* item;
	
    while(item = [enumerator nextObject]){
        [matrix addRow];
        [typeMatrix addRow];
        [[matrix cellAtRow:n column:0]setStringValue:item];
        [[typeMatrix cellAtRow:n column:0]setStringValue:
                [(NSNumber*)[_enumNames objectForKey:item]stringValue]];
        n += 1;
    }
}

@end

@implementation NSString(IDLEnum)

- (NSString*) title
{
    return self;
}

- (NSString*) stringValue
{
    return self;
}

@end

@implementation NSNumber(IDLEnum)

- (NSString*) title
{
    return [self stringValue];
}

@end
