/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLAnyObject.h"
#import "IDLDefinition_Private.h"
#import "IDLAny.h"
#import "IDLTypedef.h"

@implementation IDLAnyObject

+ (BOOL)accessInstanceVariablesDirectly
{
	return NO;
}

+ (BOOL)automaticallyNotifiesObserversForKey:(NSString *)key
{
	return NO;
}

- initWithIDLType:(id<IDLDefinition>)type
{
	self = [super init];
	_type = [(NSObject*)type retain];
	return self;
}

- init
{
	return [self initWithIDLType:nil];
}

- (void) dealloc
{
	[(NSObject*)_type release];
	[_value release];
	[super dealloc];
}

+ objectWithIDLType:(id<IDLDefinition>)type
{
	return [[[self alloc]initWithIDLType:type]autorelease];
}

- (id<IDLDefinition>) idlType
{
	return _type;
}

- (void) setIDLType:(id<IDLDefinition>)type
{
	if(type != _type){
		[self willChangeValueForKey:@"idlType"];
		[(NSObject*)_type release];
		_type = [(NSObject*)type retain];
		[self didChangeValueForKey:@"idlType"];
	}
}

- value
{
	return _value;
}

- (void) setValue:value
{
	if(value != _value){
		[self willChangeValueForKey:@"value"];
		[_value release];
		_value = [value retain];
		[self didChangeValueForKey:@"value"];
	}
}

- valueForUndefinedKey:(NSString*)key
{
	return [_value valueForKey:key];
}

- (void) setValue:value forUndefinedKey:(NSString*)key
{
	[_value setValue:value forKey:key];
}

- (NSString*) description
{
	return [NSString stringWithFormat:@"<%@ 0x%x>%@ value=%@",
		NSStringFromClass([self class]), self, [_type name], [_value description]];
}

// Forward invocation methods
// The _type may be an IDL primitive type

- (NSMethodSignature*) methodSignatureForSelector:(SEL)selector
{
    return [_value methodSignatureForSelector:selector];
}

- (void) forwardInvocation:(NSInvocation*)invocation
{
    [invocation invokeWithTarget:_value];
}

// Compatibility methods

- (id<IDLDefinition>) type
{
	NSLog(@"[IDLAnyObject type] - method is deprecated. Use method [IDLAnyObject idlType]."); 
	return [self idlType];
}
- (void) setType:(id<IDLDefinition>)type
{
	NSLog(@"[IDLAnyObject setType:] - method is deprecated. Use method [IDLAnyObject setIDLType:]."); 
	[self setIDLType:type];
}

- (BOOL) isEqual:(NSObject*)object
{
	if([object class] == isa){
		IDLAnyObject* obj = (IDLAnyObject*)object;
		return ([obj->_type referenceType] == [_type referenceType] || 
			[[obj->_type referenceType]isEqual:[_type referenceType]]) && 
			[obj->_value isEqual:_value];
	}
	return NO;
}

- (unsigned) hash
{
	return [[_type referenceType]hash] ^ [_value hash];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_value forKey:@"value"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_value];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_value = [[coder decodeObjectForKey:@"value"]retain];
	}else{
		_type = [[coder decodeObject]retain];
		_value = [[coder decodeObject]retain];
	}
	return self;
}

@end
