/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinition_Private.h"
#import "IDLPrimitive.h"
#import "CORBACoding.h"
#import "IDLInterface_Private.h"
#import "IDLTypedValue.h"
#import "IDLValueType.h"
#import "IDLOctetSequence.h"
#import "IDLOctetArray.h"
#import "IDLArrayIdentifier.h"
#import "IDLTCKind.h"
#import <expr.tab.h>

/* Filer types used to describe Ivars and Methods.  */
#define _C_ID       '@'
#define _C_CLASS    '#'
#define _C_SEL      ':'
#define _C_CHR      'c'
#define _C_UCHR     'C'
#define _C_SHT      's'
#define _C_USHT     'S'
#define _C_INT      'i'
#define _C_UINT     'I'
#define _C_LNG      'l'
#define _C_ULNG     'L'
#define _C_LNG_LNG  'q'
#define _C_ULNG_LNG 'Q'
#define _C_FLT      'f'
#define _C_DBL      'd'
#define _C_BFLD     'b'
#define _C_BOOL     'B'
#define _C_VOID     'v'
#define _C_UNDEF    '?'
#define _C_PTR      '^'
#define _C_CHARPTR  '*'
#define _C_ATOM     '%'
#define _C_ARY_B    '['
#define _C_ARY_E    ']'
#define _C_UNION_B  '('
#define _C_UNION_E  ')'
#define _C_STRUCT_B '{'
#define _C_STRUCT_E '}'
#define _C_VECTOR   '!'
#define _C_CONST    'r'

@implementation IDLPrimitive(CORBACoding)

static BOOL _mapsOctetToNSData = NO;

+ (void) setMapsOctetToNSData:(BOOL)flag
{
	_mapsOctetToNSData = flag;
}
+ (BOOL) mapsOctetToNSData
{
	return _mapsOctetToNSData;
}

- (void) encodeObject:obj withCoder:(id <CORBACoder>)inv
{
    switch(_type){
        case CHAR:
            [inv encodeChar:[obj unsignedCharValue]];
            break;
        case OCTET: // octet is mapped now to unsigned char
			if(_mapsOctetToNSData){
				const unsigned char* bytes = [obj bytes];
				[inv encodeChar:bytes[0]];
			}else
				[inv encodeChar:[obj unsignedCharValue]];
            break;
        case SHORT:
            [inv encodeShort:[obj shortValue]];
            break;
        case USHORT:
            [inv encodeUShort:[obj unsignedShortValue]];
            break;
        case INT:
            [inv encodeLong:[obj intValue]];
            break;
        case UINT:
            [inv encodeULong:[obj unsignedIntValue]];
            break;
        case LONG:
            [inv encodeLong:[obj longValue]];
            break;
        case ULONG:
            [inv encodeULong:[obj unsignedLongValue]];
            break;
        case LONGLONG:
            [inv encodeLongLong:[obj longLongValue]];
            break;
        case ULONGLONG:
            [inv encodeULongLong:[obj unsignedLongLongValue]];
            break;
        case UNSIGNED:
            [inv encodeULong:[obj unsignedIntValue]];
            break;
        case BOOLEAN:
            [inv encodeBOOL:[obj boolValue]];
            break;
        case FLOAT:
			if(obj)
				[inv encodeFloat:[obj floatValue]];
			else
				[inv encodeFloat:0.0];
            break;
        case DOUBLE:
			if(obj)
				[inv encodeDouble:[obj doubleValue]];
			else
				[inv encodeDouble:0.0];
            break;
        case STRING:
            [inv encodeString:obj];
            break;
        case WCHAR:
            [inv encodeUniChar:[obj unsignedShortValue]];
            break;
        case WSTRING:
            [inv encodeWString:obj];
            break;
        default:
            break;
    }
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)inv
{
    switch(_type){
        case CHAR:{
            unsigned char s;
            [inv decodeChar:&s];
            *obj = [NSNumber numberWithUnsignedChar:s];
            break;
        }
        case WCHAR:{
            unichar s;
            [inv decodeUniChar:&s];
            *obj = [NSNumber numberWithUnsignedShort:s];
            break;
        }
        case OCTET:{ // octet is mapped now to unsigned char
            unsigned char s;
            [inv decodeChar:&s];
			if(_mapsOctetToNSData)
				*obj = [NSData dataWithBytes:&s length:1];
			else
				*obj = [NSNumber numberWithUnsignedChar:s];
            break;
        }
        case SHORT:{
            short s;
            [inv decodeShort:&s];
            *obj = [NSNumber numberWithShort:s];
            break;
        }
        case USHORT:{
            unsigned short s;
            [inv decodeUShort:&s];
            *obj = [NSNumber numberWithUnsignedShort:s];
            break;
        }
        case INT:{
            long s;
            [inv decodeLong:&s];
            *obj = [NSNumber numberWithInt:s];
            break;
        }
        case UINT:{
            unsigned long s;
            [inv decodeULong:&s];
            *obj = [NSNumber numberWithUnsignedInt:s];
            break;
        }
        case LONG:{
            long s;
            [inv decodeLong:&s];
            *obj = [NSNumber numberWithLong:s];
            break;
        }
        case ULONG:{
            unsigned long s;
            [inv decodeULong:&s];
            *obj = [NSNumber numberWithUnsignedLong:s];
            break;
        }
        case LONGLONG:{
            long long s;
            [inv decodeLongLong:&s];
            *obj = [NSNumber numberWithLongLong:s];
            break;
        }
        case ULONGLONG:{
            unsigned long long s;
            [inv decodeULongLong:&s];
            *obj = [NSNumber numberWithUnsignedLongLong:s];
            break;
        }
        case UNSIGNED:{
            unsigned long s;
            [inv decodeULong:&s];
            *obj = [NSNumber numberWithUnsignedInt:s];
            break;
        }
        case BOOLEAN:{
            BOOL s;
            [inv decodeBOOL:&s];
            *obj = [NSNumber numberWithBool:s];
            break;
        }
        case FLOAT:{
            float s;
            [inv decodeFloat:&s];
            *obj = [NSNumber numberWithFloat:s];
            break;
        }
        case DOUBLE:{
            double s;
            [inv decodeDouble:&s];
            *obj = [NSNumber numberWithDouble:s];
            break;
        }
        case STRING:
            [inv decodeString:obj];
            break;
        case WSTRING:
            [inv decodeWString:obj];
            break;
        case VOID:
        case ONEWAY:
            *obj = nil;
            break;
        default:
            [NSException raise:CORBAMarshalException format:@"unknown primitive type"];
    }
}

- (const char*) objCType
{
    switch(_type){
        case CHAR:
            return @encode(unsigned char);
        case OCTET:
			if(_mapsOctetToNSData)
				return @encode(id);
			else
				return @encode(unsigned char);
        case SHORT:
			return @encode(short);
        case USHORT:
            return @encode(unsigned short);
        case INT:
            return @encode(int);
        case UINT:
            return @encode(unsigned int);
        case LONG:
            return @encode(long);
        case ULONG:
            return @encode(unsigned long);
        case LONGLONG:
            return @encode(long long);
        case ULONGLONG:
            return @encode(unsigned long long);
        case UNSIGNED:
            return @encode(unsigned long);
        case BOOLEAN:
            return @encode(BOOL);
        case FLOAT:
            return @encode(float);
        case DOUBLE:
            return @encode(double);
        case STRING:
            return @encode(id);
        case WSTRING:
            return @encode(id);
        case VOID:
        case ONEWAY:
            return @encode(void);
        default:
            return @encode(id);
    }
}

- (unsigned) argLength
{
    switch(_type){
        case CHAR:
            return sizeof(char);
        case OCTET:
			if(_mapsOctetToNSData)
				return sizeof(id);
			else
				return sizeof(char);
        case SHORT:
        case USHORT:
            return sizeof(short);
        case INT:
        case UINT:
        case UNSIGNED:
        case LONG:
        case ULONG:
            return sizeof(int);
        case LONGLONG:
        case ULONGLONG:
            return sizeof(long long);
        case BOOLEAN:
            return sizeof(BOOL);
        case FLOAT:
            //return sizeof(float);
            return sizeof(double);
        case DOUBLE:
            return sizeof(double);
        case STRING:
        case WSTRING:
            return sizeof(id);
        case VOID:
        case ONEWAY:
            return 0;
        default:
            return 0;
    }
}
- (unsigned) returnValueLength
{
    switch(_type){
        default:
            return [self argLength];
        case FLOAT:
            return sizeof(float);
    }
}

- (BOOL) isTypeValid:(const char*)argType
{
    switch(_type){
        case OCTET:
        case CHAR:
        case WCHAR:
        case SHORT:
        case USHORT:
        case INT:
        case UINT:
        case UNSIGNED:
        case LONG:
        case ULONG:
        case LONGLONG:
        case ULONGLONG:
        case BOOLEAN:
            return (strchr("cCBsSiIlLqQ@",argType[0]) != NULL);
        case FLOAT:
        case DOUBLE:
            return (argType[0] == _C_ID || argType[0] == _C_FLT || argType[0] == _C_DBL);
        case STRING:
            return (argType[0] == _C_ID || argType[0] == _C_CHARPTR);
        case WSTRING:
            return argType[0] == _C_ID;
        case VOID:
        case ONEWAY:
            return argType[0] == _C_VOID;
        default:
            return NO;
    }
}

// client - method signature must match IDL definition
- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    switch(_type){
        case CHAR:{
            unsigned char value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithUnsignedChar:value];
        }
        case WCHAR:{
            unsigned short value = 0;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithUnsignedShort:value];
        }
        case OCTET:{
			if(_mapsOctetToNSData){
				id value;
				[inv getArgument:&value atIndex:n];
				return value;
			}else{
				unsigned char b;
				[inv getArgument:&b atIndex:n];
				return [NSNumber numberWithUnsignedChar:b];
			}
        }
        case SHORT:{
            short value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithShort:value];
        }
        case USHORT:{
            unsigned short value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithUnsignedShort:value];
        }
        case INT:
        case LONG:{
            long value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithInt:value];
        }
        case UNSIGNED:
        case UINT:
        case ULONG:{
            unsigned long value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithUnsignedInt:value];
        }
        case LONGLONG:{
            long long value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithLongLong:value];
        }
        case ULONGLONG:{
            unsigned long long value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithUnsignedLongLong:value];
        }
        case BOOLEAN:{
            BOOL value;
            [inv getArgument:&value atIndex:n];
            return [NSNumber numberWithBool:(value?YES:NO)];
        }
        case FLOAT:{
            double fvalue; // Check
            [inv getArgument:&fvalue atIndex:n];
            return [NSNumber numberWithFloat:((float*)&fvalue)[0]];
        }
        case DOUBLE:{
            double dvalue;
            [inv getArgument:&dvalue atIndex:n];
            return [NSNumber numberWithDouble:dvalue];
        }
        case STRING:
        case WSTRING:{
            id value;
            [inv getArgument:&value atIndex:n];
            return value;
        }
        default:
            return nil;
    }
}

// Both client and server
- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
	// skip specifier encoding (r=const, n=in, N=inout, o=out, O=bycopy, V=oneway)
	while(strchr("rnNoOV", argType[0])) argType++;
    if(argType[0] == _C_PTR){
		// specifiers after _C_PTR ?
        if([self isTypeValid:argType+1]){
            switch(argType[1]){
                case _C_CHR:{ // it rather will be _C_CHARPTR
                    char* value = ptr;
					if(_type == OCTET && _mapsOctetToNSData)
						return [NSData dataWithBytes:value length:1];
					else
						return [NSNumber numberWithChar:*value];
                }
                case _C_UCHR:{ // it rather will be _C_CHARPTR
                    unsigned char* value = ptr;
					if(_type == OCTET && _mapsOctetToNSData)
						return [NSData dataWithBytes:value length:1];
					else
						return [NSNumber numberWithUnsignedChar:*value];
                }
				case _C_BOOL:{
					BOOL *value = ptr;
					return [NSNumber numberWithBool:*value];
				}
                case _C_SHT:{
                    short* value = ptr;
                    return [NSNumber numberWithShort:*value];
                }
                case _C_USHT:{
                    unsigned short* value = ptr;
                    return [NSNumber numberWithUnsignedShort:*value];
                }
                case _C_INT:{
                    int* value = ptr;
                    return [NSNumber numberWithInt:*value];
                }
                case _C_UINT:{
                    unsigned int *value = ptr;
                    return [NSNumber numberWithUnsignedInt:*value];
                }
                case _C_LNG:{
                    long *value = ptr;
                    return [NSNumber numberWithLong:*value];
                }
                case _C_ULNG:{
                    unsigned long *value = ptr;
                    return [NSNumber numberWithUnsignedLong:*value];
                }
                case _C_LNG_LNG:{
                    long long *value = ptr;
                    return [NSNumber numberWithLongLong:*value];
                }
                case _C_ULNG_LNG:{
                    unsigned long long *value = ptr;
                    return [NSNumber numberWithUnsignedLongLong:*value];
                }
                case _C_FLT:{
                    float* value = ptr;
                    return [NSNumber numberWithFloat:*value];
                }
                case _C_DBL:{
                    double* value = ptr;
                    return [NSNumber numberWithDouble:*value];
                }
                case _C_CHARPTR:{
                    const char** str = ptr;
                    return [NSString stringWithUTF8String:*str];
                }
                case _C_ID:{
                    id * value = ptr;
					return *value;
				}
            }
        }
    }else
    if(argType[0] == _C_CHARPTR  && _type == OCTET){ //  Octet -> unsigned char
        unsigned char* value = ptr;
		if(_mapsOctetToNSData)
			return [NSData dataWithBytes:value length:1];
		else
			return [NSNumber numberWithUnsignedChar:value[0]];
    }else
	if(argType[0] == _C_CHARPTR && _type == CHAR){
		char* value = ptr;
		return [NSNumber numberWithUnsignedChar:*value]; // type CHAR is treated as unsigned char
	}
    [NSException raise:CORBAInvalidArgumentException 
        format:@"invalid argument ObjCType '%s', must be '^%s'", argType, [self objCType]];
    return nil;
}

- (void) setValue:value ofType:(const char*)argType forPointer:(void*)ptr
{
	// skip specifier encoding (r=const, n=in, N=inout, o=out, O=bycopy, V=oneway)
	while(strchr("rnNoOV", argType[0])) argType++;
    if(argType[0] == _C_PTR && [self isTypeValid:argType+1]){
        switch(argType[1]){
            case _C_CHR: // it rather will be _C_CHARPTR
				if(_type == OCTET && _mapsOctetToNSData){
					const char* bytes = [value bytes];
					*((char*)ptr) = bytes[0];
				}else
					*((char*)ptr) = [value charValue];
                break;
            case _C_UCHR: // it rather will be _C_CHARPTR
				if(_type == OCTET && _mapsOctetToNSData){
					const unsigned char *bytes = [value bytes];
					*((unsigned char*)ptr) = bytes[0];
				}else
					*((unsigned char*)ptr) = [value unsignedCharValue];
                break;
			case _C_BOOL:
                *((BOOL*)ptr) = [value boolValue];
                break;
            case _C_SHT:
                *((short*)ptr) = [value shortValue];
                break;
            case _C_USHT:
                *((unsigned short*)ptr) = [value unsignedShortValue];
                break;
            case _C_INT:
                *((int*)ptr) = [value intValue];
                break;
            case _C_UINT:
                *((unsigned int*)ptr) = [value unsignedIntValue];
                break;
            case _C_LNG:
                *((long*)ptr) = [value longValue];
                break;
            case _C_ULNG:
                *((unsigned long*)ptr) = [value unsignedLongValue];
                break;
            case _C_LNG_LNG:
                *((long long*)ptr) = [value longLongValue];
                break;
            case _C_ULNG_LNG:
                *((unsigned long long*)ptr) = [value unsignedLongLongValue];
                break;
            case _C_FLT:
                *((float*)ptr) = [value floatValue];
                break;
            case _C_DBL:
                *((double*)ptr) = [value doubleValue];
                break;
            case _C_CHARPTR:
                *((const char**)ptr) = [value cString];
                break;
            case _C_ID:
                *((id*)ptr) = value;
                break;
            default:
                [NSException raise:CORBAInvalidArgumentException 
                    format:@"invalid argument type '%s', must be '^%s' or comaptible", 
                        argType, [self objCType]];
        }
    }else
    if(_type == OCTET && argType[0] == _C_CHARPTR)
    {
		if(_mapsOctetToNSData){
			const unsigned char *bytes = [value bytes];
			*((unsigned char*)ptr) = bytes[0];
		}else
			*((unsigned char*)ptr) = [value unsignedCharValue];
    }else
	if(_type == CHAR && argType[0] == _C_CHARPTR){
		*((char*)ptr) = [value unsignedCharValue];
	}else
        [NSException raise:CORBAInvalidArgumentException
            format:@"invalid argument type '%s', must be '^%s' or comaptible", 
                argType, [self objCType]];
}

- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    const char* argType = [[inv methodSignature]getArgumentTypeAtIndex:n];
	// skip specifier encoding (r=const, n=in, N=inout, o=out, O=bycopy, V=oneway
	while(strchr("rnNoOV", argType[0])) argType++;
	if([self isTypeValid:argType]){
        switch(argType[0]){
            case _C_SHT:{
                short s = [value shortValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_USHT:{
                unsigned short s = [value unsignedShortValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_INT:{
                int s = [value intValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_UINT:{
                unsigned int s = [value unsignedIntValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_LNG:{
                long s = [value longValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_ULNG:{
                unsigned long s = [value unsignedLongValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
			case _C_BOOL:{
                BOOL s = [value boolValue];
                [inv setArgument:&s atIndex:n];
			}
			break;
            case _C_CHR:{
				if(_type == OCTET && _mapsOctetToNSData){
					const char* bytes = [value bytes];
					[inv setArgument:(void*)bytes atIndex:n];
				}else{
					char s = [value charValue];
					[inv setArgument:&s atIndex:n];
				}
            }
            break;
            case _C_UCHR:{
				if(_type == OCTET && _mapsOctetToNSData){
					const unsigned char *bytes = [value bytes];
					[inv setArgument:(void*)bytes atIndex:n];
				}else{
					unsigned char s = [value unsignedCharValue];
					[inv setArgument:&s atIndex:n];
				}
            }
            break;
            case _C_LNG_LNG:{
                long long s = [value longLongValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_ULNG_LNG:{
                unsigned long long s = [value unsignedLongLongValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_FLT:{
                float s = [value floatValue]; // Check
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_DBL:{
                double s = [value doubleValue];
                [inv setArgument:&s atIndex:n];
            }
            break;
            case _C_ID:{
                id v = value;
                [inv setArgument:&v atIndex:n];
            }
            break;
            case _C_CHARPTR:{
                const char* v = [value cStringUsingEncoding:NSUTF8StringEncoding];
                [inv setArgument:&v atIndex:n];
            }
            break;
            default:
                [NSException raise:CORBAInvalidArgumentException 
                    format:@"invalid argument type '%s', must be '%s' or compatible", 
                        argType, [self objCType]];
        }
    }else
        [NSException raise:CORBAInvalidArgumentException
            format:@"invalid argument type '%s', must be '%s' or compatible", 
                argType, [self objCType]];
}

// Client side only - setting result values for invocation after invoking the method remotely
// method's return type must match one for _type
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    switch(_type){
        case CHAR:{
            unsigned char c = [value unsignedCharValue];
            [inv setReturnValue:&c];
        }
            break;
        case WCHAR:{
            unsigned short c = [value unsignedShortValue];
            [inv setReturnValue:&c];
        }
            break;
        case OCTET:{
			if(_mapsOctetToNSData){ // returns NSData*
				id val = value;
				[inv setReturnValue:&val];
			}else{
				unsigned char c = [value unsignedCharValue];
				[inv setReturnValue:&c];
			}
        }
            break;
        case SHORT:
        case USHORT:{
            short s = [value shortValue];
            [inv setReturnValue:&s];
        }
            break;
        case INT:
        case UINT:
        case UNSIGNED:
        case LONG:
        case ULONG:{
            long l = [value longValue];
            [inv setReturnValue:&l];
        }
            break;
        case LONGLONG:
        case ULONGLONG:{
            long long ll = [value longLongValue];
            [inv setReturnValue:&ll];
        }
            break;
        case BOOLEAN:{
            BOOL b = [value boolValue];
            [inv setReturnValue:&b];
        }
            break;
        case FLOAT:{
            float f = [value floatValue];
            [inv setReturnValue:&f];
        }
            break;
        case DOUBLE:{
            double d = [value doubleValue];
            [inv setReturnValue:&d];
        }
            break;
        case STRING:
        case WSTRING:{
            id _val = value;
            [inv setReturnValue:&_val];
        }
            break;
        case VOID:
        case ONEWAY:
            break;
        default:
            break;
    }
}

// Called in server - extracting result values after invoking the invocation
- returnValueFromInvocation:(NSInvocation*)inv
{
    const char* returnType = [[inv methodSignature]methodReturnType];
	// skip specifier encoding
	while(strchr("rnNoOV", returnType[0])) returnType++;
    if([self isTypeValid:returnType]){
        switch(returnType[0]){
            case _C_SHT:{
                short s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithShort:s];
            }
            case _C_USHT:{
                unsigned short s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithUnsignedShort:s];
            }
            case _C_INT:{
                int s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithInt:s];
            }
            case _C_UINT:{
                unsigned int s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithUnsignedInt:s];
            }
            case _C_LNG:{
                long s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithLong:s];
            }
            case _C_ULNG:{
                unsigned long s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithUnsignedLong:s];
            }
			case _C_BOOL:{
                BOOL b;
                [inv getReturnValue:&b];
                return [NSNumber numberWithBool:b];
			}
            case _C_CHR:{
                char s;
                [inv getReturnValue:&s];
				if(_type == OCTET && _mapsOctetToNSData)
					return [NSData dataWithBytes:&s length:1];
				else
					return [NSNumber numberWithChar:s];
            }
            case _C_UCHR:{
                unsigned char s;
                [inv getReturnValue:&s];
				if(_type == OCTET && _mapsOctetToNSData)
					return [NSData dataWithBytes:&s length:1];
				else
					return [NSNumber numberWithUnsignedChar:s];
            }
            case _C_LNG_LNG:{
                long long s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithLongLong:s];
            }
            case _C_ULNG_LNG:{
                unsigned long long s;
                [inv getReturnValue:&s];
                return [NSNumber numberWithUnsignedLongLong:s];
            }
            case _C_ID:{
                id value;
                [inv getReturnValue:&value];
                return value;
            }
            case _C_FLT:{
                float value;
                [inv getReturnValue:&value];
                return [NSNumber numberWithFloat:value];
            }
            case _C_DBL:{
                double value;
                [inv getReturnValue:&value];
                return [NSNumber numberWithDouble:value];
            }
            case _C_CHARPTR:{
                const char* value;
                [inv getReturnValue:&value];
                return [NSString stringWithUTF8String:value];
            }
            case _C_VOID:
                return nil;
            default:
                [NSException raise:CORBAInvalidArgumentException 
                    format:@"invalid return type '%s', must be '%s' or compatible", 
                        returnType, [self objCType]];
                return nil;
        }
    }else
        [NSException raise:CORBAInvalidArgumentException 
            format:@"invalid return type '%s', must be '%s' or compatible", 
                returnType, [self objCType]];
    return nil;
}

+ objectForValue:(void*)data ofKind:(TCKind)kind
{
	switch(kind){
		case tk_octet:
			if(_mapsOctetToNSData)
				return [NSData dataWithBytes:data length:1];
			else
				return [NSNumber numberWithUnsignedChar:*(unsigned char*)data];
		case tk_boolean:
			return [NSNumber numberWithBool:*(BOOL*)data];
		default:
			return nil;
	}
}

+ (void) setValue:(void*)data fromObject:object ofKind:(TCKind)kind
{
	switch(kind){
		case tk_octet:
			if(_mapsOctetToNSData){
				const unsigned char* bytes = [object bytes];
				*(unsigned char*)data = bytes[0];
			}else
			   *(unsigned char*)data = [object unsignedCharValue];
			break;
		case tk_boolean:
			*(BOOL*)data = [object boolValue];
			break;
		default:
			break;
	}
}
- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [super encodeWithCORBACoder:coder];
    switch(_type){
        case STRING:
        case WSTRING:
            [coder encodeLong:0];
            break;
        default:
            break;
    }
}

@end
