/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "CORBAObject.h"

@class VXInvocation,IDLFunction;

@interface CORBAObject(Private)

- (VXInvocation*) createInvocationFor:(IDLFunction*)fobj withArguments:(NSMutableDictionary*)args;
- (NSMutableDictionary*) invokeFunction:(IDLFunction*)fo with:(NSDictionary*)args;
+ _objectWithIORData:(NSData*)data;
- (void) addProfileWithKey:(NSData*)key host:(NSString*)hostName port:(unsigned short)port 
        version:(IDLGIOPVersion*)version;
- (void) addProfileWithKey:(NSData*)key host:(NSString*)hostName port:(unsigned short)port;
- (void) addProfileWithKey:(NSData*)key;
- (void) narrowInterface:(NSString*)typeId;
+ (CORBAObject*)uniqueObjectForObject:(CORBAObject*)obj;
- (void) setImplementation:object;
- (void) resetImplementation;
- (void) setActivated:(BOOL) flag;
- (BOOL) isActivated;
+ (CORBAObject*) lookupImplementation:impl;
+ (CORBAObject*) objectForImplementation:impl withInterface:(IDLInterface*)interface;
+ (CORBAObject*) lookupObjectID:(NSData*)objectKey;
- (NSData*)objectKey;

- (void) addProfile:profile;

+ (NSData*) objectIdForObject:object;

+ (NSString*) localHostAddress;
+ (NSSet*) localHostAddressSet;

#if !(TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE)
+ (void) setupIPMonitor;
#endif

@end

@interface CORBAObject(Naming)
- resolve:(NSArray*)nameArray;
- (void) list:(unsigned long) count bindingList:(id*)list bindingIterator:(id*)iterator;
@end

#import "CORBAException.h"

@interface CORBAException(Private)
+ exception:(NSString*)name interface:(IDLInterface*)interface;
+ exception:(NSString*)name object:(CORBAObject*)object;

@end

extern NSDictionary* decodeIIOPProfile(NSData* data);
