/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLObject.h"
#import "IDLDefinition_Private.h"
#import "IDLValueType.h"
#import "IDLTypedef.h"
#import "IDLStruct.h"

@implementation IDLObject

+ (BOOL)accessInstanceVariablesDirectly
{
	return NO;
}

+ (BOOL)automaticallyNotifiesObserversForKey:(NSString *)key
{
	return NO;
}

- initWithIDLType:(id<IDLDefinition>)type
{
	self = [super init];
	_type = [(NSObject*)type retain];
	_value = [[NSMutableDictionary alloc]init];
	return self;
}

- init
{
	return [self initWithIDLType:nil];
}

- (void) dealloc
{
	[(NSObject*)_type release];
	[_value release];
	[super dealloc];
}

+ objectWithIDLType:(id<IDLDefinition>)type
{
	return [[[self alloc]initWithIDLType:type]autorelease];
}

- (id<IDLDefinition>) idlType
{
	return _type;
}

- (void) setIDLType:(id<IDLDefinition>)type
{
	if(type != _type){
		[self willChangeValueForKey:@"idlType"];
		[(NSObject *)_type release];
		_type = [(NSObject*)type retain];
		[self didChangeValueForKey:@"idlType"];
	}
}

- (IDLValueType*) valuetype
{
	IDLDefinition* type = [(IDLDefinition*)_type referenceType];
	if([type isKindOfClass:[IDLValueType class]])
		return (IDLValueType*)type;
	else
		return nil;
}

- valueForKey:(NSString*)key
{
	return [_value valueForKey:key];
}

- (void) setValue:value forKey:(NSString*)key
{
	[self willChangeValueForKey:key];
	[_value setValue:value forKey:key];
	[self didChangeValueForKey:key];
}

- (void) removeObjectForKey:(NSString*)key
{
	NSLog(@"*** [%@ %@] - use method setValue:forKey: instead", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
	[_value setValue:nil forKey:key];
}

- (void) setObject:obj forKey:(NSString*)key
{
	NSLog(@"*** [%@ %@] - use method setValue:forKey: instead", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
	[_value setValue:obj forKey:key];
//	[self setValue:obj forKey:key];
}
- objectForKey:(NSString*)key
{
	NSLog(@"*** [%@ %@] - use method valueForKey: instead", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
	return [self valueForKey:key];
}

- (NSString*) description
{
	return [NSString stringWithFormat:@"<%@ 0x%x>%@", NSStringFromClass([self class]), self, [_type typeId]];
}

- (BOOL) isEqual:(NSObject*)object
{
	if([object isMemberOfClass:[IDLObject class]]){
		IDLObject* obj = (IDLObject*)object;
		return [obj->_value isEqual:_value];
	}
	return NO;
}

- (unsigned) hash
{
	return [_value hash];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_value forKey:@"value"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_value];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_value = [[coder decodeObjectForKey:@"value"]retain];
	}else{
		_type = [[coder decodeObject]retain];
		_value = [[coder decodeObject]retain];
	}
	return self;
}

@end
