/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "VXMessageHeader.h"
#import <IDL.h>
#import "CORBAObject.h"
#import "IDLGIOPVersion.h"

@implementation VXMessageHeader

- (unsigned) messageSize
{
    return _size;
}
- (unsigned) minorVersion
{
    return [_version minorVersion];
}
- (unsigned) majorVersion
{
    return [_version majorVersion];
}

- (IDLGIOPVersion*)iiopVersion
{
    return _version;
}
        
- (unsigned long) messageType
{
    return _messageType;
}

- (void) _encode
{
    char *magic = "GIOP";
    [_stream encodeChar:magic[0]];
    [_stream encodeChar:magic[1]];
    [_stream encodeChar:magic[2]];
    [_stream encodeChar:magic[3]];
    [_version encodeVersion:_stream];
    [_stream encodeChar:_byteOrder];
    [_stream encodeChar:_messageType];
    [_stream encodeLong:_size];
}

- (void) set
{
	_size = [_stream length] - MSG_HEADER_SIZE;
	[_stream encodeLong:_size atOffset:MSG_HEADER_SIZE_OFFSET];
}

// Create header for output
- initWithMessageType:(unsigned)msgType iiopVersion:(IDLGIOPVersion*)version stream:(IDLStream*)stream
{
    self = [super init];
    _stream = [stream retain];
    switch(NSHostByteOrder()){
        case NS_LittleEndian:
            _byteOrder = 1;
            break;
        case NS_BigEndian:
        default:
            _byteOrder = 0;
    }
    if(version)
        _version = [[IDLGIOPVersion alloc]initVersion:[version majorVersion]:[version minorVersion]];
    else
        _version = [[IDLGIOPVersion alloc]init];
    _size = 0;
    _messageType = msgType;
    [_stream setIIOPVersion:_version];
	[self _encode];
    return self;
}

- (void)dealloc
{
    [_stream release];
    [_version release];
    [super dealloc];
}
- (unsigned char) byteOrder
{
    return (_byteOrder & 1);
}

- (BOOL) moreFragments
{
    return ((_byteOrder & 2) && ([_version minorVersion] > 0)) ? YES : NO;
}

- (void) _decode
{
    unsigned char magic[4];
    _version = [[IDLGIOPVersion alloc]init];
    [_stream decodeChar:&magic[0]];
    [_stream decodeChar:&magic[1]];
    [_stream decodeChar:&magic[2]];
    [_stream decodeChar:&magic[3]];
    if(magic[0] != 'G' || magic[1] != 'I' || magic[2] != 'O' || magic[3] != 'P')
        [NSException raise:CORBACommunicationException format:@"Invalid message header magic"];
    [_version decodeVersion:_stream];
    [_stream decodeChar:&_byteOrder];
    [_stream setByteOrder:[self byteOrder]];
    [_stream decodeChar:&_messageType];
    [_stream decodeULong:&_size];
}

- initWithStream:(IDLStream*)stream
{
    self = [super init];
    _stream = [stream retain];
    [self _decode];
    [_stream setIIOPVersion:_version];
	[_stream setByteOrder:[self byteOrder]];
    return self;
}

- (NSString*) description
{
	return [NSString stringWithFormat:@"<MessageHeader: GIOP %d.%d (%@) length %d>",
		[_version majorVersion], [_version minorVersion], 
		[self byteOrder]? @"LittleEndian" : @"BigEndian", _size];
}

@end
